# -*- encoding: utf-8 -*-

from pyzrpc.rabbit.rabbit import _RabbitMQ


class RabbitMQ(_RabbitMQ):

    def config(self): super().config()

    def update(self, config): super().update(config)

    def send_message(self, queue, message): super().send_message(queue, message)

    def get_message(self, queue, callback): super().get_message(queue, callback)



# -*- encoding: utf-8 -*-
import abc


class Meta(abc.ABC):

    @property
    @abc.abstractmethod
    def rpc_proxy(self): ...

    @property
    @abc.abstractmethod
    def logger(self): ...

    @property
    @abc.abstractmethod
    def name(self): ...


class ServiceMeta(Meta):

    @property
    @abc.abstractmethod
    def service_name(self): ...

    @property
    @abc.abstractmethod
    def service_version(self): ...

    @property
    @abc.abstractmethod
    def service_ipaddr(self): ...


class WorkMeta(Meta):

    @property
    @abc.abstractmethod
    def work_name(self): ...

    @property
    @abc.abstractmethod
    def work_version(self): ...

    @property
    @abc.abstractmethod
    def work_ipaddr(self): ...



# -*- encoding: utf-8 -*-

import json
import pika
import logging

from pyzrpc.meta import SubjectMeta, CONFIG_RABBIT_CONFIG_KEY


class _RabbitMQ(SubjectMeta):
    _config = None

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value):
        self._config = value

    def update(self, config):
        self.config = config.get(CONFIG_RABBIT_CONFIG_KEY)

    def _mq_channel(self):
        host = self.config.split('@')[1].split(':')[0]
        port = self.config.split('@')[1].split(':')[1]

        user = self.config.split('@')[0].split('//')[1].split(':')[0]
        passwd = self.config.split('@')[0].split('//')[1].split(':')[1]

        credentials = pika.PlainCredentials(user, passwd)
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host, port=port, virtual_host='/', credentials=credentials, heartbeat=0))
        return connection.channel()

    def _create_queue(self, queue):
        _mq_channel = self._mq_channel()
        try:
            _mq_channel.queue_declare(queue=queue)
        except Exception as e:
            logging.error(e)
        return _mq_channel

    def send_message(self, queue, message):
        _mq_channel = self._create_queue(queue=queue)
        if type(message) is not str:
            message = json.dumps(message)
        _mq_channel.basic_publish(exchange='', routing_key=queue, body=message)

    def get_message(self, queue, callback):
        _mq_channel = self._create_queue(queue=queue)
        _mq_channel.basic_consume(on_message_callback=callback, queue=queue, auto_ack=False)
        _mq_channel.start_consuming()



# -*- encoding: utf-8 -*-
try:
    import eventlet
    eventlet.monkey_patch()
except OSError:
    pass

import os
import errno
import signal
from nameko.runners import ServiceRunner

from pyzrpc.meta import CONFIG_MONGO_FIELD_NAME_KEY, CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY, \
    CONFIG_MONGO_FIELD_SERVICE_VERSION_KEY, CONFIG_MONGO_FIELD_SERVICE_NAME_KEY, \
    CONFIG_MONGO_FIELD_SERVICE_PID_KEY, CONFIG_MONGO_FIELD_SERVICE_FUNCTIONS_KEY

from pyzrpc.observer import Observer
from pyzrpc.logger import Logger
from pyzrpc.utils import RpcProxy
from pyzrpc.mongo import DBServices

from pyzrpc.service.service_constructor import ServiceBuild


def _container_run(services, config):
    service_runner = ServiceRunner(config)
    for service_cls in services:
        service_runner.add_service(service_cls)

    def shutdown(signum, frame):
        eventlet.spawn_n(service_runner.stop)

    signal.signal(signal.SIGTERM, shutdown)

    service_runner.start()

    running = eventlet.spawn(service_runner.wait)

    while True:
        try:
            running.wait()
        except OSError as exc:
            if exc.errno == errno.EINTR:
                continue
            raise
        except KeyboardInterrupt:
            print()
            try:
                service_runner.stop()
            except KeyboardInterrupt:
                print()
                service_runner.kill()
        else:
            break


class _ServiceStart:

    @staticmethod
    def service_start(service, config):
        _observer = Observer()

        _logger = Logger()
        _rpc_proxy = RpcProxy()
        _mongo = DBServices()

        _observer.config = config
        _observer.attach(_logger)
        _observer.attach(_rpc_proxy)
        _observer.attach(_mongo)
        _observer.notify()

        build = ServiceBuild()
        _cls = build.build(cls_path=service.__file__, rpc_proxy=_rpc_proxy, logger=_logger)

        _logger.logger(_cls.service_name).info('service running : {}'.format(_cls.name))

        _mongo.update_many(
            query={
                CONFIG_MONGO_FIELD_NAME_KEY: _cls.name,
                CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY: _cls.service_ipaddr
            },
            update_data={
                CONFIG_MONGO_FIELD_NAME_KEY: _cls.name,
                CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY: _cls.service_ipaddr,
                CONFIG_MONGO_FIELD_SERVICE_NAME_KEY: _cls.service_name,
                CONFIG_MONGO_FIELD_SERVICE_VERSION_KEY: _cls.service_version,
                CONFIG_MONGO_FIELD_SERVICE_PID_KEY: os.getpid(),
                CONFIG_MONGO_FIELD_SERVICE_FUNCTIONS_KEY: _cls.functions
            },
            upsert=True
        )

        _container_run(services=[_cls], config=_rpc_proxy.rpc_config)



# -*- encoding: utf-8 -*-
import abc

from pyzrpc.service.service_start import _ServiceStart


class ServiceConstructor(abc.ABC):
    """
    RPC Metaclass Constructor
    """

    def service_name(self): ...

    def service_version(self): ...

    def service_ipaddr(self): ...

    def rpc_proxy(self): ...

    def logger(self): ...


class ServiceStart(_ServiceStart):

    def service_start(self, service, config):
        super().service_start(service, config)



# -*- encoding: utf-8 -*-
from pyzrpc.utils.utils import _Network, _FormatTime, _RpcProxy


class Network(_Network):

    def get_ipaddr(self) -> str: return super().get_ipaddr()

    def is_port_open(self, ip_addr: str, port: int) -> bool: return super().is_port_open(ip_addr, port)


class FormatTime(_FormatTime):

    def get_converted_time(self): return super().get_converted_time()


class RpcProxy(_RpcProxy):

    def rpc_config(self): super().rpc_config()

    def remote_call(self, service_name: str, method_name: str, **params):
        return super().remote_call(service_name, method_name, **params)

    def update(self, config): super().update(config)



from pyzrpc.meta.meta import ServiceMeta, WorkMeta
from pyzrpc.meta.observer import ObserverMeta, SubjectMeta

CONFIG_SELF_FUNCTION_KEY = 'self'

CONFIG_RPC_FUNCTION_KEY = 'RpcFunction'
CONFIG_WORK_FUNCTION_KEY = 'WorkFunction'
CONFIG_RPC_FUNCTION_PARAM_KEY = 'param_name'
CONFIG_RPC_FUNCTION_DEFAULT_VALUE_KEY = 'default_value'

CONFIG_RPC_FUNCTION_NAME_KEY = 'service_name'
CONFIG_WORK_FUNCTION_NAME_KEY = 'work_name'

CONFIG_RPC_PROXY_RPC_KEY = 'rpc'
CONFIG_RPC_PROXY_NAME_KEY = 'zerorpc'
CONFIG_RPC_PROXY_METHOD_NAME_KEY = 'call'

CONFIG_ROOT_PATH_KEY = 'ROOT_PATH'
CONFIG_RPC_CONFIG_KEY = 'RABBITMQ_CONFIG'
CONFIG_AMQP_URI_KEY = 'AMQP_URI'

CONFIG_SOCKET_BIND_IP = '8.8.8.8'
CONFIG_SOCKET_BIND_PORT = 80
CONFIG_SOCKET_SHUTDOWN_SLEEP = 2

CONFIG_DEFAULT_TIMEZONE = 'Asia/Shanghai'
CONFIG_DEFAULT_FORMAT = '%Y%m%d%H%M%S'

CONFIG_MONGO_CONFIG_KEY = 'MONGODB_CONFIG'
CONFIG_MONGO_MONGO_DBNAME_KEY = 'ZERORPC'

CONFIG_RABBIT_CONFIG_KEY = 'RABBITMQ_CONFIG'

CONFIG_MONGO_FIELD_NAME_KEY = 'name'
CONFIG_MONGO_FIELD_SERVICE_PID_KEY = 'service_pid'
CONFIG_MONGO_FIELD_SERVICE_NAME_KEY = 'service_name'
CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY = 'service_ipaddr'
CONFIG_MONGO_FIELD_SERVICE_VERSION_KEY = 'service_version'
CONFIG_MONGO_FIELD_SERVICE_FUNCTIONS_KEY = 'service_functions'

CONFIG_MONGO_FIELD_WORK_PID_KEY = 'work_pid'
CONFIG_MONGO_FIELD_WORK_NAME_KEY = 'work_name'
CONFIG_MONGO_FIELD_WORK_IPADDR_KEY = 'work_ipaddr'
CONFIG_MONGO_FIELD_WORK_VERSION_KEY = 'work_version'
CONFIG_MONGO_FIELD_WORK_FUNCTIONS_KEY = 'work_functions'
CONFIG_MONGO_FIELD_WORK_MAX_PROCESS_KEY = 'work_max_process'
CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY = 'work_run_process'

CONFIG_MONGO_FIELD_TASK_ID_KEY = 'task_id'



# -*- encoding: utf-8 -*-


import pytz
import socket
import datetime

from pyzrpc.meta import CONFIG_SOCKET_BIND_IP, CONFIG_SOCKET_BIND_PORT, CONFIG_SOCKET_SHUTDOWN_SLEEP, \
    SubjectMeta, CONFIG_AMQP_URI_KEY, CONFIG_DEFAULT_TIMEZONE, CONFIG_DEFAULT_FORMAT, CONFIG_RPC_CONFIG_KEY
from pyzrpc.utils.rpc_proxy import MyClusterRpcProxy


class _Network:

    @staticmethod
    def get_ipaddr() -> str:
        socket_tools = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        socket_tools.connect((CONFIG_SOCKET_BIND_IP, CONFIG_SOCKET_BIND_PORT))
        return socket_tools.getsockname()[0]

    @staticmethod
    def is_port_open(ip_addr: str, port: int) -> bool:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((ip_addr, int(port)))
            s.shutdown(CONFIG_SOCKET_SHUTDOWN_SLEEP)
            return False
        except IOError:
            return True


class _FormatTime:
    _timezone = CONFIG_DEFAULT_TIMEZONE
    _format = CONFIG_DEFAULT_FORMAT

    @property
    def timezone(self):
        return self._timezone

    @timezone.setter
    def timezone(self, value):
        self._timezone = value

    @property
    def format(self):
        return self._format

    @format.setter
    def format(self, value):
        self._format = value

    def get_converted_time(self):
        current_time = datetime.datetime.now()
        target_timezone = pytz.timezone(self.timezone)
        converted_time = current_time.astimezone(target_timezone)
        return converted_time.strftime(self.format)


class _GetClusterRpcProxy:

    @staticmethod
    def get_cluster_rpc_proxy(config: dict):
        return MyClusterRpcProxy(config)


class _RpcProxy(SubjectMeta):
    _rpc_config = None

    @property
    def rpc_config(self):
        return self._rpc_config

    @rpc_config.setter
    def rpc_config(self, value):
        self._rpc_config = value

    def remote_call(self, service_name: str, method_name: str, **params):
        rpc_obj = _GetClusterRpcProxy.get_cluster_rpc_proxy(self.rpc_config)

        obj = getattr(rpc_obj.start(), service_name)
        func = getattr(obj, method_name)
        data = func(**params)
        return data

    def update(self, config):
        self.rpc_config = {CONFIG_AMQP_URI_KEY: config.get(CONFIG_RPC_CONFIG_KEY)}



from pyzrpc.observer.observer import _Observer


class Observer(_Observer):

    def config(self): super().config()

    def attach(self, subject) -> None: super().attach(subject)

    def detach(self, subject) -> None: super().detach(subject)

    def notify(self) -> None: super().notify()



# -*- encoding: utf-8 -*-

from pyzrpc.meta import ObserverMeta


class _Observer(ObserverMeta):
    _config = None
    _observer_list = []

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value):
        self._config = value

    def attach(self, subject) -> None:
        self._observer_list.append(subject)

    def detach(self, subject) -> None:
        self._observer_list.remove(subject)

    def notify(self) -> None:
        for _observer in self._observer_list:
            _observer.update(self.config)



# -*- encoding: utf-8 -*-
import abc

from pyzrpc.rabbit import RabbitMQ
from pyzrpc.work.work_start import _WorkStart


class WorkConstructor(abc.ABC):
    """
    RPC Metaclass Constructor
    """

    def work_name(self): ...

    def work_version(self): ...

    def work_ipaddr(self): ...

    @property
    def rpc_proxy(self):
        """
            self.rpc_proxy.remote_call(
                service_name: str,
                method_name: str,
                 **params
            )
        """
        return self

    @property
    def logger(self):
        """
            logging

            self.logger.info('zerorpc')

            ...
        """
        return self

    @property
    def send_message(self) -> RabbitMQ.send_message:
        return self

    @abc.abstractmethod
    def run(self, data: dict):
        """
        Record logs：
            self.logger.info('zerorpc')

        Remote Invocation
            self.rpc_proxy.remote_call(
                service_name: str,
                method_name: str,
                 **params
            )
        """


class WorkStart(_WorkStart):

    def work_start(self, service, config): super().work_start(service, config)



# -*- encoding: utf-8 -*-
from pyzrpc.logger.logger import _Logger


class Logger(_Logger):

    def logger(self, filename: str): return super().logger(filename)

    def update(self, config: dict): super().update(config)



# -*- encoding: utf-8 -*-

import os
import sys
import inspect
import importlib
from nameko.rpc import rpc

from pyzrpc.meta import ServiceMeta, CONFIG_RPC_FUNCTION_KEY, CONFIG_SELF_FUNCTION_KEY, \
    CONFIG_RPC_FUNCTION_PARAM_KEY, CONFIG_RPC_FUNCTION_DEFAULT_VALUE_KEY, \
    CONFIG_RPC_FUNCTION_NAME_KEY, CONFIG_RPC_PROXY_NAME_KEY

from pyzrpc.logger import Logger
from pyzrpc.utils import Network, FormatTime, RpcProxy


class _ServiceConstructor(ServiceMeta):
    _name = None
    _logger = None
    _rpc_proxy = None
    _service_name = None
    _service_ipaddr = None
    _service_version = None
    _functions = None

    @property
    def functions(self):
        return self._functions

    @functions.setter
    def functions(self, value):
        self._functions = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def logger(self):
        return self._logger

    @logger.setter
    def logger(self, value):
        self._logger = value

    @property
    def rpc_proxy(self):
        return self._rpc_proxy

    @rpc_proxy.setter
    def rpc_proxy(self, value):
        self._rpc_proxy = value

    @property
    def service_name(self):
        return self._service_name

    @service_name.setter
    def service_name(self, value):
        self._service_name = value

    @property
    def service_ipaddr(self):
        return self._service_ipaddr

    @service_ipaddr.setter
    def service_ipaddr(self, value):
        self._service_ipaddr = value

    @property
    def service_version(self):
        return self._service_version

    @service_version.setter
    def service_version(self, value):
        self._service_version = value

    @classmethod
    def setattr(cls, key, value):
        setattr(cls, key, value)


class ServiceBuild:
    _constructor = _ServiceConstructor

    def build(self, cls_path: str, rpc_proxy: RpcProxy, logger: Logger):
        _script_path = os.path.dirname(cls_path)
        sys.path.insert(0, _script_path)

        _module_name, _file_extension = os.path.splitext(os.path.basename(cls_path))

        _module = __import__(
            _module_name, globals=globals(), locals=locals(),
            fromlist=[CONFIG_RPC_FUNCTION_KEY])

        importlib.reload(_module)

        _cls = getattr(_module, CONFIG_RPC_FUNCTION_KEY)

        __dict__ = _cls.__dict__
        __functions__ = {}

        for _function_name in __dict__:
            if _function_name.startswith('__') is False:
                _function = __dict__[_function_name]

                if type(_function) in [type(lambda: None)]:
                    _params = []
                    _function = rpc(_function)
                    signa = inspect.signature(_function)
                    for _name, _param in signa.parameters.items():
                        if _name != CONFIG_SELF_FUNCTION_KEY:
                            default_value = _param.default
                            if _param.default is inspect.Parameter.empty:
                                default_value = None

                            _params.append({
                                CONFIG_RPC_FUNCTION_PARAM_KEY: _name,
                                CONFIG_RPC_FUNCTION_DEFAULT_VALUE_KEY: default_value
                            })

                    __functions__.setdefault(_function_name, _params)
                self._constructor.setattr(_function_name, _function)

        self._constructor.service_name = __dict__.get(CONFIG_RPC_FUNCTION_NAME_KEY)
        self._constructor.name = '{}_{}'.format(CONFIG_RPC_PROXY_NAME_KEY, self._constructor.service_name)
        self._constructor.logger = logger.logger(self._constructor.service_name)
        self._constructor.service_ipaddr = Network().get_ipaddr()
        self._constructor.service_version = FormatTime().get_converted_time()
        self._constructor.rpc_proxy = rpc_proxy
        self._constructor.functions = __functions__

        return self._constructor



# -*- encoding: utf-8 -*-

import os
import json
import time
import multiprocessing

from pyzrpc.meta import CONFIG_MONGO_FIELD_NAME_KEY, CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY, \
    CONFIG_MONGO_FIELD_WORK_IPADDR_KEY, CONFIG_MONGO_FIELD_WORK_VERSION_KEY, \
    CONFIG_MONGO_FIELD_WORK_NAME_KEY, CONFIG_MONGO_FIELD_WORK_PID_KEY, \
    CONFIG_MONGO_FIELD_WORK_FUNCTIONS_KEY, CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY, \
    CONFIG_MONGO_FIELD_WORK_MAX_PROCESS_KEY, CONFIG_MONGO_FIELD_TASK_ID_KEY

from pyzrpc.observer import Observer
from pyzrpc.logger import Logger
from pyzrpc.utils import RpcProxy
from pyzrpc.rabbit import RabbitMQ
from pyzrpc.mongo import DBServices, DBTasks

from pyzrpc.work.work_constructor import WorkBuild

_MAX_CPU_COUNT = multiprocessing.cpu_count() - 3


class TaskRun:

    @staticmethod
    def run(cls_path: str, config: dict, boyd: dict):
        _observer = Observer()
        _observer.config = config

        _logger = Logger()
        _rpc_proxy = RpcProxy()
        _db_service = DBServices()
        _rabbitmq = RabbitMQ()

        _observer.attach(_logger)
        _observer.attach(_rpc_proxy)
        _observer.attach(_db_service)
        _observer.attach(_rabbitmq)
        _observer.notify()

        _build = WorkBuild()
        _cls = _build.build(cls_path=cls_path, rpc_proxy=_rpc_proxy, logger=_logger, rabbitmq=_rabbitmq)

        _db_service.push_one(
            query={
                CONFIG_MONGO_FIELD_WORK_NAME_KEY: _cls.work_name,
                CONFIG_MONGO_FIELD_WORK_IPADDR_KEY: _cls.work_ipaddr
            },
            update_data={
                CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY: os.getpid()
            }
        )
        _cls().run(boyd)

        _db_service.pull_one(
            query={
                CONFIG_MONGO_FIELD_WORK_NAME_KEY: _cls.work_name,
                CONFIG_MONGO_FIELD_WORK_IPADDR_KEY: _cls.work_ipaddr
            },
            update_data={
                CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY: os.getpid()
            }
        )


class MqCallback:
    _name = None
    _logger = None
    _ip_addr = None
    _db_task = None
    _db_service = None
    _rpc_proxy = None
    _cls_path = None
    _config = None

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value):
        self._config = value

    @property
    def cls_path(self):
        return self._cls_path

    @cls_path.setter
    def cls_path(self, value):
        self._cls_path = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def ip_addr(self):
        return self._ip_addr

    @ip_addr.setter
    def ip_addr(self, value):
        self._ip_addr = value

    @property
    def logger(self):
        return self._logger

    @logger.setter
    def logger(self, value):
        self._logger = value

    @property
    def db_task(self):
        return self._db_task

    @db_task.setter
    def db_task(self, value):
        self._db_task = value

    @property
    def db_service(self):
        return self._db_service

    @db_service.setter
    def db_service(self, value):
        self._db_service = value

    @property
    def rpc_proxy(self):
        return self._rpc_proxy

    @rpc_proxy.setter
    def rpc_proxy(self, value):
        self._rpc_proxy = value

    def mq_callback(self, ch, method, properties, body):
        ch.basic_ack(delivery_tag=method.delivery_tag)
        _body = json.loads(body.decode())

        if CONFIG_MONGO_FIELD_TASK_ID_KEY in _body:
            _id = _body[CONFIG_MONGO_FIELD_TASK_ID_KEY]
            if _id in self.db_service.distinct_field_query(CONFIG_MONGO_FIELD_TASK_ID_KEY, query={}):
                self.logger.logger(self.name).info(
                    '{} is stop, stop data : {}'.format(_id, _body))
            else:
                _work = self.db_service.get_all_data(
                    query={
                        CONFIG_MONGO_FIELD_WORK_NAME_KEY: self.name,
                        CONFIG_MONGO_FIELD_WORK_IPADDR_KEY: self.ip_addr
                    },
                    field={
                        '_id': 0,
                        CONFIG_MONGO_FIELD_WORK_MAX_PROCESS_KEY: 1,
                        CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY: 1
                    }
                )

                _work_max_process = _work[0].get(CONFIG_MONGO_FIELD_WORK_MAX_PROCESS_KEY)
                _work_run_process = _work[0].get(CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY)

                if len(_work_run_process) < _work_max_process and len(_work_run_process) < _MAX_CPU_COUNT:
                    multiprocessing.Process(target=TaskRun.run, args=(self.cls_path, self.config, _body,)).start()
                else:
                    time.sleep(0.2)
                    ch.basic_publish(body=body, exchange='', routing_key=self.name)
        else:
            self.logger.logger(self.name).error(
                '{} is not find, error data : {}'.format(CONFIG_MONGO_FIELD_TASK_ID_KEY, _body))


class _WorkStart:

    def work_start(self, service, config):
        _observer = Observer()
        _observer.config = config

        _logger = Logger()
        _rabbitmq = RabbitMQ()
        _rpc_proxy = RpcProxy()

        _db_task = DBTasks()
        _db_service = DBServices()

        _observer.attach(_logger)
        _observer.attach(_rabbitmq)
        _observer.attach(_rpc_proxy)
        _observer.attach(_db_task)
        _observer.attach(_db_service)
        _observer.notify()

        _build = WorkBuild()
        _cls = _build.build(cls_path=service.__file__, rpc_proxy=_rpc_proxy, logger=_logger, rabbitmq=_rabbitmq)

        _logger.logger(_cls.work_name).info('work running : {}'.format(_cls.work_name))

        _db_service.update_many(
            query={
                CONFIG_MONGO_FIELD_NAME_KEY: _cls.name,
                CONFIG_MONGO_FIELD_SERVICE_IPADDR_KEY: _cls.work_ipaddr
            },
            update_data={
                CONFIG_MONGO_FIELD_WORK_IPADDR_KEY: _cls.work_ipaddr,
                CONFIG_MONGO_FIELD_WORK_NAME_KEY: _cls.work_name,
                CONFIG_MONGO_FIELD_WORK_VERSION_KEY: _cls.work_version,
                CONFIG_MONGO_FIELD_WORK_PID_KEY: os.getpid(),
                CONFIG_MONGO_FIELD_WORK_FUNCTIONS_KEY: _cls.functions,
                CONFIG_MONGO_FIELD_WORK_RUN_PROCESS_KEY: [],
                CONFIG_MONGO_FIELD_WORK_MAX_PROCESS_KEY: 5
            },
            upsert=True
        )

        _cls_task = MqCallback()
        _cls_task.logger = _logger
        _cls_task.db_task = _db_task
        _cls_task.name = _cls.work_name
        _cls_task.rpc_proxy = _rpc_proxy
        _cls_task.db_service = _db_service
        _cls_task.ip_addr = _cls.work_ipaddr
        _cls_task.cls_path = service.__file__
        _cls_task.config = config

        while True:
            try:
                _rabbitmq.get_message(queue=_cls.work_name, callback=_cls_task.mq_callback)
            except Exception as e:
                _logger.logger(_cls.work_name).error(' {} work error : {}'.format(_cls.work_name, e))
            time.sleep(0.5)



# -*- encoding: utf-8 -*-
import pymongo

from pyzrpc.meta import SubjectMeta, CONFIG_MONGO_CONFIG_KEY, CONFIG_MONGO_MONGO_DBNAME_KEY


class Client(SubjectMeta):
    _table_name = None
    _config = None

    @property
    def table_name(self):
        return self._table_name

    @table_name.setter
    def table_name(self, value):
        self._table_name = value

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value):
        self._config = value

    def get_collection(self):
        client = pymongo.MongoClient(self.config, connect=False)
        return client[CONFIG_MONGO_MONGO_DBNAME_KEY][self.table_name]

    def update(self, config):
        self._config = config.get(CONFIG_MONGO_CONFIG_KEY)

    def get_list(self, query: dict, field: dict, limit: int, skip_no: int,
                 sort_field: str = 'update_time', sort: int = -1):
        collection = self.get_collection()
        data = collection.find(query, field).sort(sort_field, sort).limit(limit).skip(skip_no)
        data = [i for i in data]
        count = collection.count_documents(query)
        return count, data

    def update_many(self, query: dict, update_data: dict, upsert=False):
        collection = self.get_collection()
        collection.update_many(query, {"$set": update_data}, upsert=upsert)
        return None

    def push_many(self, query: dict, update_data: dict, upsert=False):
        collection = self.get_collection()
        collection.update_many(query, {"$push": update_data}, upsert=upsert)
        return None

    def push_one(self, query: dict, update_data: dict, upsert=False):
        collection = self.get_collection()
        collection.find_one_and_update(query, {"$push": update_data}, upsert=upsert)
        return None

    def pull_one(self, query: dict, update_data: dict):
        collection = self.get_collection()
        collection.find_one_and_update(query, {"$pull": update_data})
        return None

    def insert_data(self, data: dict):
        collection = self.get_collection()
        collection.insert_one(data)
        return None

    def delete_data(self, query: dict):
        collection = self.get_collection()
        collection.delete_many(query)
        return None

    def distinct_field_query(self, field: str, query: dict):
        collection = self.get_collection()
        data = collection.distinct(field, filter=query)
        return data

    def get_all_data(self, query: dict, field: dict):
        collection = self.get_collection()
        data = collection.find(query, field)
        data = [i for i in data]
        return data


class _Services(Client):

    def __init__(self):
        self.table_name = 'SERVICE_LIST'


class _Tasks(Client):

    def __init__(self):
        self.table_name = 'TASK_LIST'



from setuptools import setup, find_packages

setup(
    name='pyzrpc',
    version='1.0',
    description="pyzrpc",
    long_description=open('README.rst').read(),
    # long_description_content_type='text/plain',
    include_package_data=True,
    author='YanPing',
    author_email='zyphhxx@foxmail.com',
    maintainer='YanPing',
    maintainer_email='zyphhxx@foxmail.com',
    license='MIT License',
    url='https://gitee.com/ZYPH/zerorpc',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    python_requires=">=3.7",
    install_requires=['nameko', 'pika', 'pytz', 'pymongo', 'kombu'],
)



# -*- encoding: utf-8 -*-
import abc


class ObserverMeta(abc.ABC):

    @property
    @abc.abstractmethod
    def config(self): ...

    @abc.abstractmethod
    def attach(self, subject) -> None: ...

    @abc.abstractmethod
    def detach(self, subject) -> None: ...

    @abc.abstractmethod
    def notify(self) -> None: ...


class SubjectMeta(abc.ABC):

    @abc.abstractmethod
    def update(self, config: dict): ...



# -*- encoding: utf-8 -*-
import uuid
from kombu import Queue
from nameko.extensions import Entrypoint
from nameko.containers import WorkerContext
from nameko.standalone.rpc import PollingQueueConsumer, ConsumeEvent
from nameko.rpc import ReplyListener, RPC_REPLY_QUEUE_TEMPLATE, get_rpc_exchange, RPC_REPLY_QUEUE_TTL, MethodProxy

from pyzrpc.meta import CONFIG_RPC_PROXY_NAME_KEY, CONFIG_RPC_PROXY_METHOD_NAME_KEY


class MyReplyListener(ReplyListener):
    queue = None
    routing_key = None

    def setup(self):
        if self.queue is None:
            reply_queue_uuid = uuid.uuid4()
            service_name = self.container.service_name

            queue_name = RPC_REPLY_QUEUE_TEMPLATE.format(service_name, reply_queue_uuid)

            self.routing_key = str(reply_queue_uuid)
            exchange = get_rpc_exchange(self.container.config)
            self.queue = Queue(
                queue_name,
                exchange=exchange,
                routing_key=self.routing_key,
                queue_arguments={
                    'x-expires': RPC_REPLY_QUEUE_TTL
                },
                auto_delete=True
            )
            self.queue_consumer.register_provider(self)


class MySingleThreadedReplyListener(MyReplyListener):
    queue_consumer = None

    def __init__(self, timeout=None):
        self.queue_consumer = PollingQueueConsumer(timeout=timeout)
        super(MySingleThreadedReplyListener, self).__init__()

    def get_reply_event(self, correlation_id):
        reply_event = ConsumeEvent(self.queue_consumer, correlation_id)
        self._reply_events[correlation_id] = reply_event
        return reply_event


class MyStandaloneProxyBase(object):
    class ServiceContainer(object):
        service_name = '{}'.format(CONFIG_RPC_PROXY_NAME_KEY)

        def __init__(self, config):
            self.config = config
            self.shared_extensions = {}

    class Dummy(Entrypoint):
        method_name = CONFIG_RPC_PROXY_METHOD_NAME_KEY

    _proxy = None

    def __init__(
            self, config, context_data=None, timeout=None,
            reply_listener_cls=MySingleThreadedReplyListener
    ):
        container = self.ServiceContainer(config)
        self._worker_ctx = WorkerContext(
            container, service=None, entrypoint=self.Dummy,
            data=context_data)
        self._reply_listener = reply_listener_cls(
            timeout=timeout).bind(container)

    def __enter__(self):
        return self.start()

    def __exit__(self, tpe, value, traceback):
        self.stop()

    def start(self):
        self._reply_listener.setup()
        return self._proxy

    def stop(self):
        self._reply_listener.stop()


class MyServiceProxy(object):
    def __init__(self, worker_ctx, service_name, reply_listener, **options):
        self.worker_ctx = worker_ctx
        self.service_name = service_name
        self.reply_listener = reply_listener
        self.options = options

    def __getattr__(self, name):
        return MethodProxy(
            self.worker_ctx,
            self.service_name,
            name,
            self.reply_listener,
            **self.options
        )


class MyServiceRpcProxy(MyStandaloneProxyBase):
    def __init__(self, service_name, *args, **kwargs):
        super(MyServiceRpcProxy, self).__init__(*args, **kwargs)
        self._proxy = MyServiceProxy(self._worker_ctx, service_name, self._reply_listener)


class MyClusterProxy(object):
    def __init__(self, worker_ctx, reply_listener):
        self._worker_ctx = worker_ctx
        self._reply_listener = reply_listener
        self._proxies = {}

    def __getattr__(self, name):
        if name not in self._proxies:
            _service_proxy = MyServiceProxy(self._worker_ctx, name, self._reply_listener)
            self._proxies[name] = _service_proxy
        return self._proxies[name]

    def __getitem__(self, name):
        return getattr(self, name)


class MyClusterRpcProxy(MyStandaloneProxyBase):
    def __init__(self, *args, **kwargs):
        super(MyClusterRpcProxy, self).__init__(*args, **kwargs)
        self._proxy = MyClusterProxy(self._worker_ctx, self._reply_listener)



# -*- encoding: utf-8 -*-
import os
import uuid
import logging
from logging import handlers

from pyzrpc.meta import SubjectMeta, CONFIG_ROOT_PATH_KEY


class _Logger(SubjectMeta):
    _logger = None
    _logs_path = None
    format_str = logging.Formatter('%(asctime)s - %(levelname)s: %(message)s (%(filename)s:%(lineno)d)')

    @property
    def logs_path(self):
        return self._logs_path

    @logs_path.setter
    def logs_path(self, value):
        self._logs_path = value

    def logger(self, filename):
        if self._logger is None:
            self._logger = logging.getLogger(str(uuid.uuid4()))
            self._logger.setLevel(logging.INFO)

            th = handlers.TimedRotatingFileHandler(
                filename=f'{self.logs_path}/{filename}', when='MIDNIGHT', backupCount=7, encoding='utf-8')
            th.suffix = "%Y-%m-%d.log"
            th.setFormatter(self.format_str)

            ch = logging.StreamHandler()
            ch.setFormatter(self.format_str)

            self._logger.addHandler(th)
            self._logger.addHandler(ch)
        return self._logger

    def update(self, config):
        self.logs_path = os.path.join(config.get(CONFIG_ROOT_PATH_KEY), 'zero_rpc_logs')
        os.makedirs(self.logs_path, exist_ok=True)
